/*
  ktracker (c) 2006 Kris Beevers

  This file is part of ktracker.

  ktracker is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  ktracker is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with ktracker; if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

  $Id: perspective.cpp,v 1.1.1.1 2006/10/10 20:41:29 beevek Exp $
*/

#include <inttypes.h>
#include <stdio.h>
#include <math.h>
#include <SDL/SDL.h>
#include <opencv/cv.h>

CvPoint2D32f img_pts[4];

void compute_transforms()
{
  // assume the user clicks on ground points in the right order and
  // that the ground points are in fact (0,0), (1,0), (1,1), and (0,1)
  CvPoint2D32f ground_pts[4];
  ground_pts[0] = cvPoint2D32f(0,0);
  ground_pts[1] = cvPoint2D32f(1,0);
  ground_pts[2] = cvPoint2D32f(1,1);
  ground_pts[3] = cvPoint2D32f(0,1);

  CvMat *p_img2ground = cvCreateMat(3, 3, CV_32FC1);
  cvWarpPerspectiveQMatrix(img_pts, ground_pts, p_img2ground);
  for(uint32_t i = 0; i < 3; ++i)
    for(uint32_t j = 0; j < 3; ++j)
      printf("%g ", cvmGet(p_img2ground, i, j));
  printf("\n");

  CvMat *p_ground2img = cvCreateMat(3, 3, CV_32FC1);
  cvWarpPerspectiveQMatrix(ground_pts, img_pts, p_ground2img);
  for(uint32_t i = 0; i < 3; ++i)
    for(uint32_t j = 0; j < 3; ++j)
      printf("%g ", cvmGet(p_ground2img, i, j));
  printf("\n");
}

extern bool load_ppm(const char *file, uint8_t **grid, uint32_t *w, uint32_t *h);

int main(int argc, char **argv)
{
  if(argc < 2) {
    printf("Usage: %s input.ppm\n", argv[0]);
    return 1;
  }

  uint8_t *in;
  uint32_t inw, inh;
  if(!load_ppm(argv[1], &in, &inw, &inh)) {
    perror("load_ppm()");
    return 1;
  }

  if(SDL_Init(SDL_INIT_VIDEO) < 0) {
    perror("SDL_Init()");
    return 1;
  }

  SDL_Surface *screen = SDL_SetVideoMode(inw, inh, 24, SDL_SWSURFACE);
  if(!screen) {
    perror("SDL_SetVideoMode()");
    return 1;
  }

  memcpy(screen->pixels, in, inw*inh*3);
  SDL_UpdateRect(screen, 0, 0, 0, 0);
  IplImage *ipl = cvCreateImageHeader(cvSize(inw,inh), IPL_DEPTH_8U, 3);
  ipl->origin = 0;
  cvSetData(ipl, screen->pixels, inw*3);

  uint32_t img_count = 0;

  SDL_Event sdlevent;
  while(SDL_WaitEvent(&sdlevent) != 0) {
    if(sdlevent.type == SDL_QUIT ||
       (sdlevent.type == SDL_KEYDOWN && sdlevent.key.keysym.sym == SDLK_q))
      break;
    if(sdlevent.type != SDL_MOUSEBUTTONDOWN || img_count == 4)
      continue;
    img_pts[img_count++] = cvPoint2D32f(double(sdlevent.button.x), double(sdlevent.button.y));

    cvCircle(ipl, cvPointFrom32f(img_pts[img_count-1]), 3, CV_RGB(0,255,0), CV_FILLED);
    if(img_count > 1)
      cvLine(ipl, cvPointFrom32f(img_pts[img_count-1]), cvPointFrom32f(img_pts[img_count-2]),
	     CV_RGB(0,255,0));
    if(img_count == 4)
      cvLine(ipl, cvPointFrom32f(img_pts[3]), cvPointFrom32f(img_pts[0]), CV_RGB(0,255,0));
    SDL_UpdateRect(screen, 0, 0, 0, 0);

    if(img_count == 4)
      compute_transforms();
  }

  return 0;
}
